
/*******************************************************************************

    ig.h
    10 Mar 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or 
    (at your option) any later version.

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#ifndef IG_H
#define IG_H

#define IG     "IG v1.1"
#define IGlong "Imitation Game & Fixed Point library, version 1.1"

typedef enum
  {
  igFALSE = 0,
  igTRUE = 1 
  }
igBOOL;

/* On error, we 'abnormally end' the programme. */

void igAbend(char *file, int line, char *msg);

/*******************************************************************************
We keep track of the seed to the random generator, to allow a run (eg, the same 
function or the same sequence of guesses) to be repeated.  The igSeedRand() 
function should be called before any calls to the rand() library function are 
made.  If it's argument is 0, a 'random' seed will be generated.  The igNormal()
function returns a normally distributed random variable with given mean and 
standard deviation (ie, square_root(variance)).  
*******************************************************************************/

extern unsigned int igRandomSeed;

void igSeedRand(unsigned int);

double igNormal(double mean, double sd);

/*******************************************************************************
This group of functions return vectors, of dimension "dim", of doubles.  For
igGetNVect(), the doubles are drawn from the normal distribution with the 
nominated mean & SD.  For igGetUVect(), the doubles are picked uniformly at 
random from the closed interval [min,max].  For igGetCVect(), the doubles are 
the constant "c", while for igGetVect(), the doubles are uninitialised.  For 
igReadVect(), the doubles are read from stdin.  igCopyVector() clones the data
in a vector.
NB: In all cases, it is the user's responsibility to free the returned vector. 
*******************************************************************************/

double* igGetNVect(int dim, double mean, double sd);
double* igGetUVect(int dim, double min, double max);

double* igGetCVect(int dim, double c);
double* igGetVect(int dim);

double* igReadVect(int dim);
double* igCopyVect(int dim, double *v);

/*******************************************************************************
The L_1, L_2 and L_infinity norms, on the difference between 2 vectors, are 
supported.  TBA: If "v2" is not given (ie, NULL), then return the norm of "v1"?
*******************************************************************************/

double igL1Norm(int dim, double *v1, double *v2);
double igL2Norm(int dim, double *v1, double *v2);
double igLinfNorm(int dim, double *v1, double *v2);

/* Generic pretty-print routine for vectors. */

void igPrettyP(int dim, double *v, char *s);

/* Routines to check and/or to enforce whether vectors do or do not have various
properties. */

igBOOL igVectFinite(int dim, double *v);
igBOOL igClipVect(int dim, double *v, double min, double max);
igBOOL igOnBound(int dim, double *v, double min, double max, double tol);

#endif

