
/*******************************************************************************

    igfp.h
    2 Jun 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or 
    (at your option) any later version.

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#ifndef IGFP_H
#define IGFP_H

#include "ig.h"
#include "iglist.h"

/* These are the various parameters for the fixed-point problem being solved,
along with the function and the norm to be used. */

extern int igDim;        /* dimension of X & Y spaces */
extern double igTol;     /* tolerance on the X-Y norms */
extern double igZero;    /* tolerance on zero values or differences */
extern int igPLimit;     /* limit on number of Lemke-Howson pivots */

typedef enum
  {
  igLHlast,    /* use the last (most recent) X on the list as initial pivot */
  igLHnorm,    /* use X from list with min'm X-Y norm as initial pivot */
  igLHbest     /* use strategy yielding best oldX-newY norm as initial pivot */
  }
igLHPIV;

extern igLHPIV igLHPivot;

typedef igBOOL (*igFUN)(double *x, double *y);
extern igFUN igFunction;

typedef double (*igNORM)(int d, double *x, double *y);
extern igNORM igNorm;

/* The possible status returns from one of the FP routines.  Note that these all
take an igList argument, and the status reflects whether or not 1 (or more) 
X/Y/norm triples were appended to the list.  If an FP (or an approx'n) was 
found, it will be the last item on the list. */

typedef enum
  {
  igFP,      /* fixed-point (within tolerance) was found & appended to list */
  igOK,      /* no (within tolerance) FP found, but list was updated */
  igFail     /* unspecified problem, list was not updated */
  }
igFPSTAT;

igFPSTAT igGuess(igList *igl, int cnt, double rmin, double rmax);
igFPSTAT igIterate(igList *igl, int cnt);
igFPSTAT igImitate(igList *igl, int cnt, unsigned int debug);

#endif


