
/*******************************************************************************

    iglist.c
    22 Apr 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "ig.h"
#include "iglist.h"

#include <stdio.h>
#include <stdlib.h>

#define CHUNK 16

igList* igNewList(int dim)
  {
  igList *igl;

  if ((igl = (igList*)malloc(sizeof(igList))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewList()"); }

  igl->dim = dim;

  igl->alloc = CHUNK;
  igl->size = 0;

  if ((igl->dx = (double**)malloc(igl->alloc * sizeof(double*))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewList()"); }

  if ((igl->dy = (double**)malloc(igl->alloc * sizeof(double*))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewList()"); }

  if ((igl->dn = (double*)malloc(igl->alloc * sizeof(double))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewList()"); }

  return igl;
  }

/* NB: This frees all the used X & Y vectors, all the allocated space, and the 
struct itself. */

void igFreeList(igList *igl)
  {
  int i;
  if (igl == NULL) { return; }
  for (i = 0; i < igl->size; i++) 
    { free(igl->dx[i]);  free(igl->dy[i]); }
  free(igl->dx);
  free(igl->dy);
  free(igl->dn);
  free(igl);
  }

/* This shrinks the list size to s, while leaving the allocated space unchanged.
NB: This discards (ie, frees) the data (if any) in indices from [s] up. */

void igShrinkList(igList *igl, int s)
  {
  int i;

  if (s < 0 || s > igl->size)
    { igAbend(__FILE__, __LINE__, "bad new size in igShrinkList()"); }

  for (i = s; i < igl->size; i++)
    { free(igl->dx[i]);  free(igl->dy[i]); }

  igl->size = s;
  }

/* NB: As it stands, "x" & "y" could be non-NULL; should we disallow this? */

void igAppend(igList *igl, double *x, double *y, double n)
  {
  if (igl->size < 0 || igl->size > igl->alloc)
    { igAbend(__FILE__, __LINE__, "invalid size value in igAppend()"); }

  if (igl->size == igl->alloc)
    {
    igl->dx = (double**)
                realloc((void*)(igl->dx), (igl->alloc+CHUNK) * sizeof(double*));
    if (igl->dx == NULL)
      { igAbend(__FILE__, __LINE__, "can't realloc in igAppend()"); }

    igl->dy = (double**)
                realloc((void*)(igl->dy), (igl->alloc+CHUNK) * sizeof(double*));
    if (igl->dy == NULL)
      { igAbend(__FILE__, __LINE__, "can't realloc in igAppend()"); }

    igl->dn = (double*)
                 realloc((void*)(igl->dn), (igl->alloc+CHUNK) * sizeof(double));
    if (igl->dn == NULL)
      { igAbend(__FILE__, __LINE__, "can't realloc in igAppend()"); }

    igl->alloc += CHUNK;
    }

  /* We fall through to here if there's already enough space. */

  igl->dx[igl->size] = x;
  igl->dy[igl->size] = y;
  igl->dn[igl->size] = n;

  igl->size++;
  }


