
/*******************************************************************************

    iglist.h
    14 Apr 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

A simple implementation of an expandable list of (X, Y, norm) tuples.  The
dimension is fixed at initialisation and thereafter as many tuples as required
can be appended to the list.  Space is allocated in chunks, and a new chunk is 
allocated as required when tuples are appended.

*******************************************************************************/

#ifndef IGLIST_H
#define IGLIST_H

typedef struct
  {
  int dim;             /* dimension of the point data */
  int alloc, size;     /* allocated & used space */
  double **dx;         /* the X point data */
  double **dy;         /* the Y point data */
  double *dn;          /* the norm data */
  }
igList;

igList* igNewList(int dim);
void igFreeList(igList *igl);
void igShrinkList(igList *igl, int s);

void igAppend(igList *igl, double *x, double *y, double n);

#endif


