
/*******************************************************************************

    igmatrix.c
    27 Apr 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "igmatrix.h"

#include <stdio.h>
#include <stdlib.h>

#define CHUNK 16

igMatrix* igNewMatrix(void)
  {
  int i;
  igMatrix *igm;

  if ((igm = (igMatrix*)malloc(sizeof(igMatrix))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewMatrix()"); }

  igm->alloc = CHUNK;
  igm->size = 0;

  if ((igm->dpp = (double**)malloc(igm->alloc * sizeof(double*))) == NULL)
    { igAbend(__FILE__, __LINE__, "can't malloc in igNewMatrix()"); }

  for (i = 0; i < igm->alloc; i++)
    {
    if ((igm->dpp[i] = (double*)malloc(igm->alloc * sizeof(double))) == NULL)
      { igAbend(__FILE__, __LINE__, "can't malloc in igNewMatrix()"); }
    }

  igm->mset = igFALSE;

  return igm;
  }

/* NB: This frees the struct itself, as well as all its allocated space. */

void igFreeMatrix(igMatrix *igm)
  {
  int i;
  if (igm == NULL) { return; }
  for (i = 0; i < igm->alloc; i++)
    { free(igm->dpp[i]); }
  free(igm->dpp);
  free(igm);
  }

/* Increase the 'used' size of the array by one, preserving any existing data.
NB: The added row/col, and any new space (re)allocated, are not initialised. */

void igResize(igMatrix *igm)
  {
  int i;

  if (igm->size < 0 || igm->size > igm->alloc)
    { igAbend(__FILE__, __LINE__, "invalid size value in igResize()"); }

  if (igm->size == igm->alloc)
    {
    igm->dpp = (double**)
               realloc((void*)(igm->dpp), (igm->alloc+CHUNK) * sizeof(double*));
    if (igm->dpp == NULL)
      { igAbend(__FILE__, __LINE__, "can't realloc in igResize()"); }

    for (i = 0; i < igm->alloc; i++)
      {
      igm->dpp[i] = (double*)
             realloc((void*)(igm->dpp[i]), (igm->alloc+CHUNK) * sizeof(double));
      if (igm->dpp[i] == NULL)
        { igAbend(__FILE__, __LINE__, "can't realloc in igResize()"); }
      }

    for (i = igm->alloc; i < igm->alloc+CHUNK; i++)
      {
      igm->dpp[i] = (double*)malloc((igm->alloc+CHUNK) * sizeof(double));
      if (igm->dpp[i] == NULL)
        { igAbend(__FILE__, __LINE__, "can't malloc in igResize()"); }
      }

    igm->alloc += CHUNK;
    }

  /* We fall through to here if there's already enough space. */

  igm->size++;
  }

/*******************************************************************************
Although the array's elements can be set directly, they should be set by this
routine so that the minimum & maximum values are tracked.  NB: Resetting an 
already set element is possible, but not recommended, since it may not properly 
track the min/max entries.  The i & j indicex values are not checked.
*******************************************************************************/

void igSetIJ(igMatrix *igm, int i, int j, double d)
  {
  igm->dpp[i][j] = d;
  if (igm->mset)
    {
    if (d < igm->min) { igm->min = d; }
    if (d > igm->max) { igm->max = d; }
    }
  else
    { igm->min = igm->max = d;  igm->mset = igTRUE; }
  }


