
/*******************************************************************************

    igmatrix.h
    22 Apr 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

A simple implementation of an expandable 2D square array of doubles.  For use
with imitation games, the struct is used to store the -||Xi-Yi||^2 values so we 
don't have to keep recalculating them as the game size grows.  The dpp[][] array
is used by the Lemke-Howson code to build the linear complementarity problem to 
be solved.

The space is allocated in chunks, and a new chunk is added as required when the
matrix is resized (ie, the actual used space, initially none, is increased).  In
order that things work correctly, and that the max & min entries are properly
tracked, after each igResize() call all of the entries in the new row & col 
should be explicitly set (once only!) using igSetIJ().

*******************************************************************************/

#ifndef IGMATRIX_H
#define IGMATRIX_H

#include "ig.h"

typedef struct
  {
  int alloc, size;     /* allocated & used space */
  double **dpp;        /* data */
  double min, max;     /* running min/max values */
  igBOOL mset;         /* min & max have been init'd */
  }
igMatrix;

igMatrix* igNewMatrix(void);
void igFreeMatrix(igMatrix *igm);

void igResize(igMatrix *igm);
void igSetIJ(igMatrix *igm, int i, int j, double d);

#endif


