
/*******************************************************************************

    igtableau.h
    17 Mar 11
    Colin Ramsay
    Copyright (C) 2011, The Gambit Project

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

The igtableau.* source files were originally created by concatenating a set of 
C source files which implemented a stand-alone version of the Lemke-Howson
algorithm and a 'look-alike' of the Gambit "lcp" utility.  (This code was based 
on original code by Codenotti, DeRossi & Pagan (21 Aug 06), as modified by me 
(11 Aug 09)).  They were then ruthlessly edited to excise the unused code, to 
add some new code, to trap all possible (floating-point induced) errors in the
Lemke-Howson routine, and to integrate them into my fixed-point via imitation 
games code.  Although we only use the Lemke-Howson routine for imitation games, 
it can cope with arbitrary (incl non-square) games, it need not be started at 
the null equilibrium, and any initial pivot can be used.  The returned Nash
equilibrium (if it's not the artifical one) may have all-zero probability 
vectors due to floating-point problems, so this needs to be checked.  The 
initial tableau (which represents the null, or artifical, equilibrium) is 
created from an igMatrix struct and, if this has been `properly' set up, is 
guaranteed to have non-negative finite entries.

*******************************************************************************/

#ifndef IGTABLEAU_H
#define IGTABLEAU_H

#include "igmatrix.h"

typedef struct
  {
  double ***tab;     /* the 2 sub-tableaux (2D arrays) for the X & Y players */
  int dim1, dim2;    /* the number of strategies of players 1 & 2 (ie, X & Y) */
  int *ind;          /* array for lexico-maximal check */
  int numP;          /* number of pivot operations */
  double zero;       /* the zero tolerance to use */
  }
igTableau;

igTableau* igNewTableau(igMatrix *igm, double tol);
void igFreeTableau(igTableau *igt);

/* These are all the things that can happen when Lemke-Howson is called. */

typedef enum
  {
  igNotInit,     /* tableau not in initial state */
  igBadArg,      /* bad initial pivot or pivot limit argument */
  igPivLim,      /* pivot limit exceeded */
  igNoRow,       /* can't find positive coefficient row */
  igLexMax,      /* can't find lexico-maximal row */
  igNE           /* reached a Nash equilibrium */
  }
igLHSTAT;

igLHSTAT igLemHow(igTableau *igt, int initP, int limP);

double* igGetXProb(igTableau *igt);
double* igGetYProb(igTableau *igt);

int igGetPivots(igTableau *igt);

void igDumpTab(igTableau *igt);

#endif


